<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2006
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*****
* Word Processor class
*
* @author Mark Grimshaw
* 
* $Header: /cvsroot/wikindx/wikindx3/core/wp/WP.php,v 1.46 2006/02/01 20:40:29 sirfragalot Exp $
*****/
class WP
{
// Constructor
	function WP($db, $vars)
	{
		$this->db = $db;
		$this->vars = $vars;
		include_once("core/session/SESSION.php");
		$this->session = new SESSION();
		include_once("core/messages/MESSAGES.php");
		$this->messages = new MESSAGES();
		include_once("core/messages/ERRORS.php");
		$this->errors = new ERRORS();
		include_once("core/messages/SUCCESS.php");
		$this->success = new SUCCESS();
		include_once("core/messages/UTF8.php");
		$this->utf8 = new UTF8();
		include_once("core/template/TEMPLATE.php");
		$this->template = new TEMPLATE('content');
// All papers are stored in the papers/ folder
		$this->dirName = "papers";
	}
/*
* Check we are allowed to perform SUWP operations
* @author Mark Grimshaw
* @version 1
* @param string function name
* @return string
*/
	function gateKeep($function)
	{
		if(!$this->session->getVar('setup_write'))
		{
			include_once("core/authorize/AUTHORIZE.php");
			$authorize = new AUTHORIZE($this->db);
			return $authorize->initLogon();
		}
// Attempt a browser sanity check since HTML design mode is not available on all browsers.
// $_SERVER[HTTP_USER_AGENT]:
// Opera 8.5 gives 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; en) Opera 8.50' - not compatible.
// Konqueror gives 'Mozilla/5.0 (compatible; Konqueror/3.2; Linux) (KHTML, like Gecko)' -- not compatible.
//
// Mozilla 1.7.8 gives'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.7.8) Gecko/20050603 Fedora/1.7.8-1.2.1.legacy' -- compatible.
// Netscape 8 gives 'Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.7.5) Gecko/20050729 Netscape/8.0.3.3' - compatible.
// Firefox gives 'Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.7.12) Gecko/20050915 Firefox/1.0.7' - compatible.
// IE 6 gives 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 1.1.4322)' - compatible.
		if(!$this->session->getVar("paper_browserCompatibility"))
		{
			$browserIdent = $_SERVER['HTTP_USER_AGENT'];
			if(preg_match("/Gecko|Mozilla\/4\.0 .*?compatible; MSIE 6\.0;/", $browserIdent) && 
			!preg_match("/Konqueror|Opera/", $browserIdent))
				$this->session->setVar("paper_browserCompatibility", TRUE); // compatible.
// Probably incompatible browser but user wants to try anyway
			else if($function == 'wpBrowserTry')
			{
				$this->session->setVar("paper_browserCompatibility", TRUE); // compatible or they want to try it anyway.
				$this->displayWp();
			}
			else
				return $this->displayWarning();
		}
		if($function == 'wpDisplay')
			$this->displayWp();
// new paper
		else if($function == 'wpNew')
		{
			$this->session->clearArray("wp");
			$pString = $this->newWp();
		}
// save paper
		else if($function == 'wpSave')
			$pString = $this->saveWp();
// revert to saved paper - ask for confirmation
		else if($function == 'wpRevertConfirm')
			$pString = $this->revertConfirmWp();
// revert to saved paper
		else if($function == 'wpRevert')
			$pString = $this->revertWp();
// list papers for opening one
		else if($function == 'wpListOpen')
			$pString = $this->listOpenWp();
// list for downloading/backing up.
		else if($function == 'wpList')
			$pString = $this->listWp();
		else if($function == 'wpFileDownload')
			$pString = $this->fileDownloadWp();
		else if($function == 'wpExportDownload')
			$pString = $this->exportDownloadWp();
// list papers for deleting one or more
		else if($function == 'wpListDelete')
			$pString = $this->listDeleteWp();
//confirm delete papers
		else if($function == 'wpDeleteConfirm')
			$pString = $this->deleteConfirmWp();
// delete paper(s)
		else if($function == 'wpDelete')
			$pString = $this->deleteWp();
// open paper
		else if($function == 'wpOpen')
			$pString = $this->openWp();
// import paper - init
		else if($function == 'wpImportInit')
			$pString = $this->importInitWp();
// import paper
		else if($function == 'wpImport')
			$pString = $this->importWp();
		$this->template->setVar('body', $pString);
		return $this->template->process();
	}
/*
* Display a warning in case of a probably browser incompatibility
* @author Mark Grimshaw
* @version 1
*/
	function displayWarning()
	{
		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		$pString = $this->errors->text('warning', 'suwpBrowserWarning');
		$pString .= MISC::p(MISC::a("link", $this->messages->text('wp', 'browserTry'), 
			"index.php?action=wpBrowserTry"));
		$this->template->setVar('body', $pString);
		return $this->template->process();
	}
/*
* Display SUWP options
* @author Mark Grimshaw
* @version 1
* @param string Error message
* @return string
*/
	function displayWp($message = FALSE)
	{
		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		if($message)
			$this->template->setVar('body', MISC::p($message));
		include_once("core/html/MENU.php");
		$menu = new MENU();
		$this->template->setVar('noMenu', $menu->noMenu('wp'));
		$pString = $this->template->process();
		include_once("core/html/CLOSE.php");
		new CLOSE($this->db, $pString);
	}
/*
* Form for inporting paper
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function importInitWp()
	{
		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		$pString = MISC::p($this->messages->text("wp", "import2"));
		$pString .= FORM::formMultiHeader("wpImport");
		$pString .= MISC::p(FORM::fileUpload($this->messages->text("import", "file"), "import_file", 30));
		$pString .= MISC::p(FORM::formSubmit('Proceed'));
		$pString .= FORM::formEnd();
		return $pString;
	}
/*
* Import paper
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function importWp()
	{
		if(!isset($_FILES['import_file']))
			$this->badInput($this->errors->text("file", "upload"), 'displayWp');
		$userId = $this->session->getVar('setup_userId');
// Check for file input
		if(!($text = @file_get_contents($_FILES['import_file']['tmp_name'])))
			$this->badInput($this->errors->text("file", "read"), "displayWp");
		$text = trim($text);
// Remove any extension
		$split = explode(".", $_FILES['import_file']['name']);
		if(sizeof($split) > 1)
		{
			array_pop($split);
			$title = join(".", $split);
		}
		else
			$title = $_FILES['import_file']['name'];
		$fileName = sha1($userId . $title . $text);
		$fullFileName = $this->dirName . "/" . $fileName;
// If file exists, we don't import
		if(file_exists($fullFileName))
			$this->badInput($this->errors->text("warning", "paperExists"), 'displayWp');
// Else, write file and database entry
		if(!move_uploaded_file($_FILES['import_file']['tmp_name'], $fullFileName))
			$this->badInput($this->errors->text("file", "upload"));
		$fields[] = 'hashFilename';
		$values[] = $fileName;
		$fields[] = 'userId';
		$values[] = $userId;
		$fields[] = 'filename';
		$values[] = $title;
		$fields[] = 'timestamp';
		$values[] = $this->db->formatTimestamp();
		$this->db->insert('WKX_papers', $fields, $values);
		$databaseId = $this->db->lastAutoId();
		$this->session->setVar("wp_text", base64_encode(stripslashes($text)));
		$this->session->setVar("wp_title", base64_encode(stripslashes($title)));
		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
// Include JavaScript file
		include_once("core/wp/JsFormatBar/FORMATBAR.php");
		$pString = MISC::jsExternal('core/wp/JsFormatBar/JavaScript/JavaScript.js');
		$pString .= MISC::p($this->success->text("importWp"));
		$pString .= FORMATBAR::printFB($fileName, $databaseId);
		return $pString;
	}
/*
* List papers ready to delete one or more
* @author Mark Grimshaw
* @version 1
* @param string Error message
* @return string
*/
	function listDeleteWp($error = FALSE)
	{
		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		$pString = '';
		if($error)
			$pString = $error;
		list($papers, $papersTimestamp) = $this->grabPapers();
		if(!empty($papers))
		{
			foreach($papers as $id => $title)
				$entries[$id] = $title . " (" . $papersTimestamp[$id] . ")";
			$pString .= FORM::formHeader("wpDeleteConfirm");
			$pString .= FORM::selectFBoxValueMultiple(FALSE, "id", $entries, 20, 80) . 
				MISC::br() . $this->messages->text("hint", "multiples") . MISC::br() . 
				MISC::br() . FORM::formSubmit('Confirm');
			$pString .= FORM::formEnd();
		}
		else
			$pString = MISC::p($this->messages->text("wp", "noPapers"));
		return $pString;
	}
/*
* Ask for confirmation of delete papers
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function deleteConfirmWp()
	{
		if(!array_key_exists('id', $this->vars))
			$this->badInput($this->errors->text("inputError", "missing"), 'listDeleteWp');
		$this->session->delVar("lockDeleteWp");
		$this->template->setVar("heading", $this->messages->text("heading", "wp"));
		$userId = $this->session->getVar('setup_userId');
		include_once("core/html/HTML.php");
		$html = new HTML();
		$pString = FORM::formHeader("wpDelete");
		foreach($this->vars['id'] as $id)
		{
			$conditions[] = $this->db->tidyInput($id);
			$pString .= FORM::hidden("paperDelete_" . $id, $id);
		}
		$condition = join(" OR " . $this->db->formatField('id') . "=", $conditions);
// We do a further check here that these files really do belong to the user.
		$recordSet = $this->db->select(array("WKX_papers"), array("id", "filename"), 
			" WHERE (" . $this->db->formatField('id') . "=" . $condition . 
			") AND " . $this->db->formatField('userId') . '=' . $this->db->tidyInput($userId));
		while($row = $this->db->loopRecordSet($recordSet))
			$papers[] = $html->dbToHtmlTidy($row['filename']);
		$paper = join(", ", $papers);
		$pString .= MISC::p($this->messages->text("wp", "deleteConfirm", ": $paper"));
		$pString .= MISC::br() . FORM::formSubmit('Confirm');
		$pString .= FORM::formEnd();
		return $pString;
	}
/*
* Delete papers(s)
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function deleteWp()
	{
		if($this->session->getVar("lockDeleteWp"))
			$this->badInput($this->errors->text("done", "wp"), 'displayWp');
		$userId = $this->session->getVar('setup_userId');
		$ids = array();
		foreach($this->vars as $key => $value)
		{
			if(!$value)
				continue;
			if(!preg_match("/paperDelete_/", $key, $matches))
				continue;
			$ids[] = $value;
		}
		if(empty($ids))
			$this->badInput($this->errors->text("inputError", "missing"), 'listDeleteWp');
		foreach($ids as $id)
		{
			$recordSet = $this->db->select(array("WKX_papers"), array('hashFilename'), 
			" WHERE " . $this->db->formatField('id') . '=' . $this->db->tidyInput($id) . 
			" AND " . $this->db->formatField('userId') . '=' . $this->db->tidyInput($userId));
			$file = $this->dirName . "/" . $this->db->fetchOne($recordSet);
			@unlink($file);
			$this->db->delete("WKX_papers", 
			" WHERE " . $this->db->formatField('id') . '=' . $this->db->tidyInput($id) . 
			" AND " . $this->db->formatField('userId') . '=' . $this->db->tidyInput($userId));
		}
		$this->db->optimize("WKX_papers");
		$pString = $this->success->text("deleteWp");
		$this->session->setVar("lockDeleteWp", TRUE);
		return $this->displayWp($pString);
	}
/*
* List papers for downloading/backup
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function listWp()
	{
		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		list($papers, $papersTimestamp) = $this->grabPapers(TRUE);
		if(!empty($papers))
		{
			foreach($papers as $id => $paperArray)
			{
				$hash = $paperArray[0];
				$title = $paperArray[1];
				$timestamp = $papersTimestamp[$id];
				$fileLinks[] = MISC::a("link", $title, 
					"index.php?action=wpFileDownload" . htmlentities("&hash=$hash&filename=$title"), "_new") . 
					"&nbsp;&nbsp;&nbsp;($timestamp)";
			}
			$pString = MISC::p($this->messages->text("wp", "backup"));
			$pString .= join(MISC::br(), $fileLinks);
		}
		else
			$pString = MISC::p($this->messages->text("wp", "noPapers"));
		return $pString;
	}
/*
* Download a file providing the real filename instead of the hash filename
* @author Mark Grimshaw
* @version 1
*/
	function fileDownloadWp()
	{
		if(!array_key_exists('hash', $this->vars))
			$this->badInput($this->errors->text("inputError", "missing"), 'displayWp');
		$fileName = $this->vars['filename'];
		ob_start();
		header( "Content-Disposition: attachment; filename=\"$fileName\";");
		header("Expires:  0");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header("Pragma: public");
		readfile($this->dirName . "/" . $this->vars['hash']);
		ob_end_flush();
		die;
	}
/*
* Download a paper export providing the real filename instead of the hash filename
* @author Mark Grimshaw
* @version 1
*/
	function exportDownloadWp()
	{
		if(!array_key_exists('hash', $this->vars))
			$this->badInput($this->errors->text("inputError", "missing"));
		$extension = $this->vars['extension'];
		$fileName = $this->vars['filename'];
		if($this->vars['extension'] == 'rtf')
			$type = 'text/rtf';
		ob_start();
		header("Content-type: $type");
		header( "Content-Disposition: inline; filename=\"$fileName\";" );
		header("Expires:  0");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header("Pragma: public");
		readfile("files/" . $this->vars['hash'] . "." . $extension);
		ob_end_flush();
		die;
	}
/*
* List papers ready to open one
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function listOpenWp()
	{
		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		$pString = '';
		list($papers, $papersTimestamp) = $this->grabPapers();
		if(!empty($papers))
		{
			foreach($papers as $id => $title)
				$entries[$id] = $title . " (" . $papersTimestamp[$id] . ")";
			$pString .= FORM::formHeader("wpOpen");
			$pString .= FORM::selectFBoxValue(FALSE, "id", $entries, 20, 80) . 
				MISC::br() . FORM::formSubmit('Select');
			$pString .= FORM::formEnd();
		}
		else
			$pString = MISC::p($this->messages->text("wp", "noPapers"));
		return $pString;
	}
/*
* Display blank new WP interface
* @author Mark Grimshaw
* @version 1
* @param string Error message
* @return string
*/
	function newWp($message = FALSE)
	{
		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
// Include JavaScript file
		include_once("core/wp/JsFormatBar/FORMATBAR.php");
		$pString = MISC::jsExternal('core/wp/JsFormatBar/JavaScript/JavaScript.js');
		if($message)
			$pString .= $message;
		$pString .= FORMATBAR::printFB();
		return $pString;
	}
/*
* Revert to saved paper - ask for confirmation first
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function revertConfirmWp()
	{
		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		if(!array_key_exists('databaseID', $this->vars))
			$this->badInput($this->errors->text("inputError", "missing"), "displayWp");
		$pString = MISC::p($this->messages->text("wp", "revertConfirm"));
		$pString .= FORM::formHeader("wpRevert");
		$pString .= FORM::hidden("databaseID", $this->vars['databaseID']);
		$pString .= MISC::p(FORM::formSubmit('Confirm'));
		$pString .= FORM::formEnd();
		return $pString;
	}
/*
* Revert to saved paper
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function revertWp()
	{
		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		if(!array_key_exists('databaseID', $this->vars))
			$this->badInput($this->errors->text("inputError", "missing"), "displayWp");
		$recordSet = $this->db->select(array("WKX_papers"), array("hashFilename", "filename"), 
			" WHERE " . $this->db->formatField("id") . "=" . $this->db->tidyInput($this->vars['databaseID']));
		$row = $this->db->fetchRow($recordSet);
		$fullFileName = $this->dirName . '/' . $row['hashFilename'];
		$title = stripslashes($row['filename']);
		if(!($text = @file_get_contents($fullFileName)))
			$this->badInput($this->errors->text("file", "read", ": $fullFileName"), "displayWp");
		$text = $this->utf8->decodeUtf8($text);
		$this->session->setVar("wp_text", base64_encode($text));
		$this->session->setVar("wp_title", base64_encode($title));
// Include JavaScript file
		include_once("core/wp/JsFormatBar/FORMATBAR.php");
		$pString = MISC::jsExternal('core/wp/JsFormatBar/JavaScript/JavaScript.js');
		$pString .= FORMATBAR::printFB($row['hashFilename'], $this->vars['databaseID']);
		return $pString;
	}
/*
* Open existing paper
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function openWp()
	{
		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		if(!array_key_exists('id', $this->vars))
			$this->badInput($this->errors->text("inputError", "missing"), "displayWp");
		$recordSet = $this->db->select(array("WKX_papers"), array("hashFilename", "filename"), 
			" WHERE " . $this->db->formatField("id") . "=" . $this->db->tidyInput($this->vars['id']));
		$row = $this->db->fetchRow($recordSet);
		$fullFileName = $this->dirName . '/' . $row['hashFilename'];
		$title = stripslashes($row['filename']);
		if(!($text = @file_get_contents($fullFileName)))
			$this->badInput($this->errors->text("file", "read", ": $fullFileName"), "displayWp");
		$text = $this->utf8->decodeUtf8($text);
		$this->session->setVar("wp_text", base64_encode($text));
		$this->session->setVar("wp_title", base64_encode($title));
// Include JavaScript file
		include_once("core/wp/JsFormatBar/FORMATBAR.php");
		$pString = MISC::jsExternal('core/wp/JsFormatBar/JavaScript/JavaScript.js');
		$pString .= FORMATBAR::printFB($row['hashFilename'], $this->vars['id']);
		return $pString;
	}
/*
* Save paper
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function saveWp()
	{
		$this->template->setVar('heading', $this->messages->text("heading", "wp"));
		if(!$this->writeSession())
			$this->badInput($this->errors->text("inputError", "missing"), "newWp");
		$text = stripslashes(trim($this->vars['hdnpaperText']));
		$text = $this->tidyHtml($text);
		$title = trim($this->vars['title']);
		$userId = $this->session->getVar('setup_userId');
		$fileName = sha1($userId . $title . $text);
// inserting
		if(!array_key_exists('databaseID', $this->vars))
		{
			$fields[] = 'hashFilename';
			$values[] = $fileName;
			$fields[] = 'userId';
			$values[] = $userId;
			$fields[] = 'filename';
			$values[] = $title;
			$fields[] = 'timestamp';
			$values[] = $this->db->formatTimestamp();
			$this->db->insert('WKX_papers', $fields, $values);
			$databaseId = $this->db->lastAutoId();
		}
// updating
		else
		{
			$updateArray['hashFilename'] = $fileName;
			$updateArray['filename'] = $title;
			$updateArray['timestamp'] = $this->db->formatTimestamp();
			$databaseId = $this->vars['databaseID'];
			$this->db->update('WKX_papers', $updateArray, 
				" WHERE " . $this->db->formatField('id') . "=" . $this->db->tidyInput($databaseId));
		}
		$fullFileName = $this->dirName . '/' . $fileName;
		if(!$fp = fopen("$fullFileName", "w"))
			$this->badInput($this->errors->text("file", "write", ": $fileName"), "newWp");
		if(!fputs($fp, $this->utf8->encodeUtf8($text)))
			$this->badInput($this->errors->text("file", "write", ": $fileName"), "newWp");
		fclose($fp);
// if this is a re-save, remove old hashed file from folder  if it's not the same
		if(array_key_exists('fileName', $this->vars) && 
			($this->vars['fileName'] != $fileName) && 
			file_exists($this->dirName . '/' . $this->vars['fileName']))
				unlink($this->dirName . '/' . $this->vars['fileName']);
// Include JavaScript file
		include_once("core/wp/JsFormatBar/FORMATBAR.php");
		$pString = MISC::jsExternal('core/wp/JsFormatBar/JavaScript/JavaScript.js');
		$pString .= MISC::p($this->success->text("saveWP"));
// do we also want to export to RTF?
		if($exportRtfLink = $this->exportRtf($fileName, $title, $text))
			$pString .= $exportRtfLink;
		$pString .= FORMATBAR::printFB($fileName, $databaseId);
		return $pString;
	}
/*
* Attempt to tidy up HTML
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function tidyHtml($text)
	{
// Nested DIVS occur (in Mozilla) usually where a user selects all text across multiple paragraphs, aligns that and then proceeds to align internal paragraphs.
		$textArray = preg_split("/(<div.*[>]+.*)/Usi", 
			$text, -1, PREG_SPLIT_DELIM_CAPTURE|PREG_SPLIT_NO_EMPTY);
		if(sizeof($textArray) > 2)
			$text = $this->tidyDiv($textArray);
// Convert multiple whitespace characters to one
		$text = preg_replace("/ {2}/s", ' ', $text);
		return $text;
	}
/*
* Attempt to tidy up nested DIVs
* @author Mark Grimshaw
* @version 1
* @return string
*/
	function tidyDiv($textArray)
	{
		$nestCleaned = FALSE;
		foreach($textArray as $index => $element)
		{
			if(!$index || !($index %2))
			{
				if(!$nestCleaned)
					$lastElement = $textArray[$index];
				continue;
			}
			if(!preg_match("/<\/div>/si", $element))
			{
				$textArray[$index] .= "</div>";
				$nestCleaned = TRUE;
			}
			else if(preg_match_all("/<\/div>/si", $element, $null) > 1)
			{
				$textArray[$index] = preg_replace("/<\/div>/si", "</div>$lastElement", 
					$textArray[$index], 1);
				$nestCleaned = FALSE;
			}
		}
		return join($textArray);
	}
/*
* Get all this user's papers' titles and ids
* @author Mark Grimshaw
* @version 1
* @param string BOOLEAN
* @return array
*/
	function grabPapers($download = FALSE)
	{
		$deleted = FALSE;
		include_once("core/html/HTML.php");
		$html = new HTML();
		$papers = $papersTimestamp = $hashCheck = $deleteMe = array();
		$fields = array("id", "hashFilename", "filename", "timestamp");
		$userId = $this->session->getVar('setup_userId');
		$recordSet = $this->db->select(array("WKX_papers"), $fields, 
			" WHERE " . $this->db->formatField("userId") . "=" . $this->db->tidyInput($userId) . 
			" ORDER BY " . $this->db->formatField('timestamp') . " DESC");
		while($row = $this->db->loopRecordSet($recordSet))
		{
			if($download)
			{
				$papers[$row['id']][0] = $row['hashFilename'];
				$papers[$row['id']][1] = $html->dbToFormTidy($row['filename']);
				$papersTimestamp[$row['id']] = $html->dbToFormTidy($row['timestamp']);
// In case of multiple entries for same hashed file (see below)
				if(array_key_exists($row['hashFilename'], $hashCheck))
					$deleteMe[] = $row['id'];
				else
					$hashCheck[$row['hashFilename']] = TRUE;
			}
			else
			{
				$papers[$row['id']] = $html->dbToFormTidy($row['filename']);
				$papersTimestamp[$row['id']] = $html->dbToFormTidy($row['timestamp']);
// In case of multiple entries for same hashed file (see below)
				if(array_key_exists($row['hashFilename'], $hashCheck))
					$deleteMe[] = $row['id'];
				else
					$hashCheck[$row['hashFilename']] = TRUE;
			}
// There's a bug somewhere I haven't been able to track down that saves multiple versions of the same hashed file in the db table.  This is a fix to prevent 
// the user noticing if another db entry of the same hash filename has previously been deleted...
			$fullFileName = $this->dirName . '/' . $row['hashFilename'];
// Remove db entries if file no longer exists
			if(!file_exists("$fullFileName"))
			{
				$this->db->delete("WKX_papers", " WHERE " . $this->db->formatField('hashFilename') . '=' . 
					$this->db->tidyInput($row['hashFilename']));
				$deleted = TRUE;
			}
		}
// If there are multiple db entries of the same hash filename but the file exists, remove all but the latest from the database
		foreach($deleteMe as $id)
		{
			$this->db->delete("WKX_papers", " WHERE " . $this->db->formatField('id') . '=' . 
				$this->db->tidyInput($id));
			$deleted = TRUE;
		}
		if($deleted)
			$this->db->optimize("WKX_papers");
		return array($papers, $papersTimestamp);
	}
/*
* export paper to RTF
* @author Mark Grimshaw
* @version 1
* @param string hashed filename
* @param string real filename
* @param string input text
* @return string hyperlink to RTF file
*/
	function exportRtf($fileName, $title, $text)
	{
		if(!array_key_exists('exportPaper_format', $this->vars))
			return FALSE;
		else
			$this->session->setVar('exportPaper_format', $this->vars['exportPaper_format']);
		if($this->vars['exportPaper_format'] == 'noExport')
			return FALSE;
		if(array_key_exists('exportPaper_style', $this->vars))
			$this->session->setVar('exportPaper_style', $this->vars['exportPaper_style']);
		if(array_key_exists('exportPaper_spacePaper', $this->vars))
			$this->session->setVar('exportPaper_spacePaper', $this->vars['exportPaper_spacePaper']);
		else
			$this->session->setVar('exportPaper_spacePaper', 'singleSpace');
		if(array_key_exists('exportPaper_spaceBib', $this->vars))
			$this->session->setVar('exportPaper_spaceBib', $this->vars['exportPaper_spaceBib']);
		else
			$this->session->setVar('exportPaper_spaceBib', 'singleSpace');
		if(array_key_exists('exportPaper_indentBib', $this->vars))
			$this->session->setVar('exportPaper_indentBib', $this->vars['exportPaper_indentBib']);
		else
			$this->session->setVar('exportPaper_indentBib', 'none');
		if(array_key_exists('exportPaper_spaceFt', $this->vars))
			$this->session->setVar('exportPaper_spaceFt', $this->vars['exportPaper_spaceFt']);
		else
			$this->session->setVar('exportPaper_spaceFt', 'singleSpace');
		if(array_key_exists('exportPaper_indentFt', $this->vars))
			$this->session->setVar('exportPaper_indentFt', $this->vars['exportPaper_indentFt']);
		else
			$this->session->setVar('exportPaper_indentFt', 'none');
		include_once("core/wp/WPRTF.php");
		$this->rtf = new WPRTF();
		$output = $this->rtf->openRtf();
		$text = $this->rtfParse($text);
		if(isset($this->rtf->fontBlock))
			$output .= $this->rtf->fontBlock;
		unset($this->rtf->fontBlock);
		$output .= $this->styleSheet;
// Set default font and size and black colour and left justification
		$output .= "{\\f0\\fs24\n\\s1\\cf1\\ql\\li1\\fi1\n";
		$output .= ltrim($text);
		$output .= "\n\\par\n}";
		$output .= $this->rtf->closeRtf();
// Ensure there are no hanging HTML tags (some browsers produce untidy HTML which wikindx cannot handle)
// This removed as some bib. styles want <URL> in the bibliography for web articles (e.g. MLA and WIKINDX styles).
//		$output = preg_replace("/<.*>/Usi", '', $output);
//print "$output<P>";
// Write to files/ folder
		$fullFileName = 'files/' . $fileName . '.rtf';
		if(!$fp = fopen("$fullFileName", "w"))
			$this->badInput($this->errors->text("file", "write", ": $fileName"), "newWp");
		if(!fputs($fp, $this->rtf->utf8_2_unicode($output)))
			$this->badInput($this->errors->text("file", "write", ": $fileName"), "newWp");
		fclose($fp);
// Write exported hashed filename to session
		if($sessVar = $this->session->getVar('paperExports'))
			$sessArray = unserialize($sessVar);
		else
			$sessArray = array();
		if(array_search($fileName, $sessArray) === FALSE)
		{
			$sessArray[$fileName] = $title . '.rtf';
			$this->session->setVar('paperExports', serialize($sessArray));
		}
// Perform some system admin
		include_once("core/file/FILE.php");
		$file = new FILE($this->db);
		$file->tidyFiles();
// Create download link and return it
		$titleUrl = rawurlencode($title);
		return MISC::p(MISC::a("link", $title . '.rtf', "index.php?action=wpExportDownload" . 
			htmlentities("&hash=$fileName&filename=$titleUrl&extension=rtf"), "_new"));
	}
/*
* parse BBCode and wikindx [...] code to RTF tags
* @author Mark Grimshaw
* @version 1
* @param string input text
* @return string parsed text
*/
	function rtfParse($text)
	{
		include_once("core/cite/CITE.php");
		$cite = new CITE($this->db, $this->vars);
// Deal with potential RTF control characters first
		$text = str_replace("\\", "\\\\", stripslashes($text));
		$text = str_replace("{", "\\{", $text);
		$text = str_replace("}", "\\}", $text);
// Replace CR and newline characters and combinations with a single space
		$text = preg_replace("/(\r\n)+|(\n\r)+|\n+|\r+/", ' ', $text);
// Deal with citations
		$text = $cite->parseCitations($text, 'rtf', FALSE);
// If we have no cite tags to parse, $cite->cite is not set but we still need the stylesheet so have to load CITESTYLE and CITEFORMAT anyway
		if(!$cite->cite)
		{
			include_once("core/styles/CITESTYLE.php");
			$cite = new CITESTYLE($this->db, 'rtf');
			$this->styleSheet = $cite->citeformat->styleSheet;
		}
		else
			$this->styleSheet = $cite->cite->citeformat->styleSheet;
		unset($cite);
// Simple substitutions - italics and bold sometimes come in as standard HTML tags if they were copy 'n' pasted from elsewhere.
		$pattern = array(
//							"/\s*<br>\s*<\/p>/si",
//							"/\s*(.*?)\s*<br>/si",
							"/<p>\s*(.*?)\s*<\/p>\s*/si",
							"/<sup>(.*?)<\/sup>/si",
							"/<sub>(.*?)<\/sub>/si",
							"/&amp;/si",
							"/\s*<br.*?>\s*/si",
							"/<em>(.*?)<\/em>/si",
							"/<i>(.*?)<\/i>/si",
							"/<strong>(.*?)<\/strong>/si",
							"/<b>(.*?)<\/b>/si",
							"/&lt;/si",
							"/&gt;/si",
						);
		$change = array(
//							"</p>",
//							"\\s1\\cf1$1__WIKINDX__NEWLINEPAR__", // temporary replace
							"\\s1\\cf1$1__WIKINDX__NEWLINEPAR____WIKINDX__NEWLINEPAR__", // temporary replace
							"{\\super $1}",
							"{\\sub $1}",
							"&",
							"__WIKINDX__NEWLINEPAR__",
							"{\\i $1}",
							"{\\i $1}",
							"{\\b $1}",
							"{\\b $1}",
							"\\u60 ?",
							"\\u62 ?",
						);
		$text = preg_replace($pattern, $change, $text);
// handle <font>...</font>
		$text = $this->rtf->parseFont($text);
// handle <span>...</span>
		$text = $this->rtf->parseSpan($text);
		$text = $this->rtf->createLists($text);
// Handle tables
		$text = $this->rtf->createTables($text);
// Handle images
		$text = preg_replace_callback("/<img.*>/Usi",
			array($this->rtf, "imageCallback"), $text);
// URL + emails
		$text = $this->rtf->createEmail($text);
		$text = $this->rtf->createFancyUrl($text);
// Handle <div>...</div>
		$text = $this->rtf->parseDiv($text);
// Replace temporary newlines
		$text = str_replace("__WIKINDX__NEWLINEPAR__", "\\par\n", $text);
		$text = str_replace("__WIKINDX__NEWLINE__", "\n", $text);
		$text = str_replace("&nbsp;", " ", $text);
// when user is cut 'n' pasting, superfluous codes are sometimes inserted so remove them
		$text = preg_replace("/[ ]*<span.*[>]+(.*)<\/span[>]+[ ]*/Usi", "$1", $text);
		return $text;
	}
/*
* write input to session
* @author Mark Grimshaw
* @version 1
* @return BOOLEAN
*/
	function writeSession()
	{
		$missing = FALSE;
		if(!array_key_exists('hdnpaperText', $this->vars) || !trim($this->vars['hdnpaperText']))
			$missing = TRUE;
		else
			$this->session->setVar("wp_text", base64_encode(stripslashes(trim($this->vars['hdnpaperText']))));
		if(!array_key_exists('title', $this->vars) || !trim($this->vars['title']))
			$missing = TRUE;
		else
			$this->session->setVar("wp_title", base64_encode(stripslashes(trim($this->vars['title']))));
		if($missing)
			return FALSE;
		return TRUE;
	}
/*
* Error handling
* @author Mark Grimshaw
* @version 1
* @param string Error message
* @param string Class method
*/
	function badInput($error, $method = FALSE)
	{
		$pString = $this->$method($error);
		$this->template->setVar('body', $pString);
		$pString = $this->template->process();
		include_once("core/html/CLOSE.php");
		new CLOSE($this->db, $pString);
	}
}
?>
