<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2004
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*****
* CONSTANTS_ptbr wikindx class - BRAZILIAN PORTUGUESE.
*
* Various numbers, days, months etc.
* 
*	NOTE TO TRANSLATORS:	Only translate between START TRANSLATION and END TRANSLATION tags.
*
*****/
class CONSTANTS_ptbr
{
// Constructor
	function CONSTANTS_ptbr()
	{
// Separator between a resource's title and subtitle for bibliographic styles
		$this->titleSubtitleSeparator = ": ";
// When using the word processor how are extra, unnamed authors indicated?  In English this would be 'et al.' as in:
// 'According to Grimshaw et al., "blah blah blah".
		$this->textEtAl = 'et al.';
// When using the word processor, how is singular possessive defined.  This is only for a single creator's surname and, like 'at al.' above, is used 
// to detect if a creator name should be removed from the in-text citation if that name is in the same sentence as the citation.
// If this value contains an apostrophe (as in the English Grimshaw's), then it should be in double quotes (or else the PHP script will fail).
// English has two forms (the second below for names that end in 's').
// If there is no equivalent possessive form in another language, set these to FALSE.
		$this->possessive1 = FALSE;
		$this->possessive2 = FALSE;
	}
// Convert cardinal to ordinal numbers.
// The coding here is likely to be different for each language.  If there is no logic that can be programmed for your language, then it is recommended that
// you use English ordinals as given here so that something is printed and the script does not fail.
// $modulo holds the remainder of $cardinal/100 or $cardinal/10.
// Optional $field allows conditions based on gender.  Currently, $field will be either 'edition' or 'dayMonth' (day of the month).  Additionally, $field can be used 
// to determine whether ordinals will be used at all.  For example, if this is French and $field == 'dayMonth', ordinals are used solely for '1er'.  
// (languages/fr/CONSTANTS.php has an example using $field.)
//  The French equivalent to this function would be:
/*
	function cardinalToOrdinal($cardinal, $field = FALSE)
	{
		if(($field == 'dayMonth') && ($cardinal != 1))
			return $cardinal; // no change
		$modulo = $cardinal % 10;
		if($modulo == 1) // 1st
		{
			if($field == 'edition')
				return $cardinal . 'ère';
			return $cardinal . 'er';
		}
		else // all other numbers
			return $cardinal . 'ème';
	}
*/
	function cardinalToOrdinal($cardinal, $field = FALSE)
	{
		$modulo = $cardinal % 100;
		if(($modulo == 11) || ($modulo == 12) || ($modulo == 13)) // 11th, 12th and 13th
			return $cardinal . 'th';
		$modulo = $cardinal % 10;
		if(($modulo >= 4) || !$modulo) // all other numbers
			return $cardinal . 'th';
		if($modulo == 1) // 1st
			return $cardinal . 'st';
		if($modulo == 2) // 2nd
			return $cardinal . 'nd';
		if($modulo == 3) // 3rd
			return $cardinal . 'rd';
	}
// convert cardinal numbers
	function convertNumbers($number = FALSE, $convert = FALSE)
	{
		if(!is_numeric($number))
			return $number;
// month number to long name	
		$longMonth = $this->monthToLongName();
// short month
		$shortMonth = $this->monthToShortName();
// cardinal -> ordinal word
		$ordinalWord = $this->cardinalToOrdinalWord();
// arabic -> roman numerals
		$roman = $this->cardinalToRoman();
/****************************
 START TRANSLATION
****************************/
// arabic -> ordinal (e.g. 3 -> 3rd, 10 -> 10th)
// Usually used for edition numbers in bibliographic styles that require e.g. '10th edition' rather than 'edition 10' - superseded by cardianlToOrdinal() function above
// A maximum of 50 seems a reasonable number to go up to....
// If necessary, you may need to add more loops or individual array elements.
		for($i = 4; $i <= 50; $i++)
			$ordinal[$i] = $i . 'º';
/****************************
 END TRANSLATION
****************************/
// arabic -> cardinal (i.e. no change)
		$cardinal = range(0, 50);
// !$number, we are simply loading the arrays for use elsewhere, so return
		if(!$number)
			return;
// get rid of leading '0' if necessary
		if($number < 10)
			$number += 0;
// If that number is not actually in array, we return the number as is
		if(array_key_exists($number, ${$convert}))
			return ${$convert}[$number];
		return $number;
	}
// Convert month to long name
	function monthToLongName()
	{
		return array(
/****************************
 START TRANSLATION
****************************/
				1	=>	'Janeiro',
				2	=>	'Fevereiro',
				3	=>	'Março',
				4	=>	'Abril',
				5	=>	'Maio',
				6	=>	'Junho',
				7	=>	'Julho',
				8	=>	'Agosto',
				9	=>	'Setembro',
				10	=>	'Outubro',
				11	=>	'Novembro',
				12	=>	'Dezembro',
/****************************
 END TRANSLATION
****************************/
			);
	}
// Convert month to short name
	function monthToShortName()
	{
		return array(
/****************************
 START TRANSLATION
****************************/
				1	=>	'Jan',
				2	=>	'Fev',
				3	=>	'Mar',
				4	=>	'Abr',
				5	=>	'Mai',
				6	=>	'Jun',
				7	=>	'Jul',
				8	=>	'Ago',
				9	=>	'Set',
				10	=>	'Out',
				11	=>	'Nov',
				12	=>	'Dez',
/****************************
 END TRANSLATION
****************************/
			);
	}
// convert cardinal to roman
	function cardinalToRoman()
	{
// arabic -> roman numerals
// Do NOT translate
		return array(
				1	=>	'I',
				2	=>	'II',
				3	=>	'III',
				4	=>	'IV',
				5	=>	'V',
				6	=>	'VI',
				7	=>	'VII',
				8	=>	'VIII',
				9	=>	'IX',
				10	=>	'X',
				11	=>	'XI',
				12	=>	'XII',
				13	=>	'XIII',
				14	=>	'XIV',
				15	=>	'XV',
				16	=>	'XVI',
				17	=>	'XVII',
				18	=>	'XVIII',
				19	=>	'XIX',
				20	=>	'XX',
				21	=>	'XXI',
				22	=>	'XXII',
				23	=>	'XXIII',
				24	=>	'XXIV',
				25	=>	'XXV',
				26	=>	'XXVI',
				27	=>	'XXVII',
				28	=>	'XXVIII',
				29	=>	'XXIX',
				30	=>	'XXX',
				31	=>	'XXXI',
				32	=>	'XXXII',
				33	=>	'XXXIII',
				34	=>	'XXXIV',
				35	=>	'XXXV',
				36	=>	'XXXVI',
				37	=>	'XXXVII',
				38	=>	'XXXVIII',
				39	=>	'XXXIX',
				40	=>	'XXXX',
				41	=>	'XXXXI',
				42	=>	'XXXXII',
				43	=>	'XXXXIII',
				44	=>	'XXXXIV',
				45	=>	'XXXXV',
				46	=>	'XXXXVI',
				47	=>	'XXXXVII',
				48	=>	'XXXXVIII',
				49	=>	'XXXXIX',
				50	=>	'L',
			);
	}
// convert ordinal to word
	function cardinalToOrdinalWord()
	{
		return array(
/****************************
 START TRANSLATION
****************************/
// Usually used for edition numbers in bibliographic styles that require words rather than arabic numerals.
// Any numbers not listed here will be returned without change.  e.g. here, 51 will be returned as 51 rather than 'fifty-first'.
// A maximum of 50 seems a reasonable number to go up to....
						'1'	=>	'Primeiro',
						'2'	=>	'Segundo',
						'3'	=>	'Terceiro',
						'4'	=>	'Quarto',
						'5'	=>	'Quinto',
						'6'	=>	'Sexto',
						'7'	=>	'Sétimo',
						'8'	=>	'Oitavo',
						'9'	=>	'Nono',
						'10'	=>	'Décimo',
						'11'	=>	'Décimo primeiro',
						'12'	=>	'Décimo segundo',
						'13'	=>	'Décimo terceiro',
						'14'	=>	'Décimo quarto',
						'15'	=>	'Décimo quinto',
						'16'	=>	'Décimo sexto',
						'17'	=>	'Décimo sétimo',
						'18'	=>	'Décimo oitavo',
						'19'	=>	'Décimo nono',
						'20'	=>	'Vigésimo',
						'21'	=>	'Vigésimo primeiro',
						'22'	=>	'Vigésimo segundo',
						'23'	=>	'Vigésimo terceiro',
						'24'	=>	'Vigésimo quarto',
						'25'	=>	'Vigésimo quinto',
						'26'	=>	'Vigésimo sexto',
						'27'	=>	'Vigésimo sétimo',
						'28'	=>	'Vigésimo oitavo',
						'29'	=>	'Vigésimo nono',
						'30'	=>	'Trigésimo',
						'31'	=>	'Trigésimo primeiro',
						'32'	=>	'Trigésimo segundo',
						'33'	=>	'Trigésimo terceiro',
						'34'	=>	'Trigésimo quarto',
						'35'	=>	'Trigésimo quinto',
						'36'	=>	'Trigésimo sexto',
						'37'	=>	'Trigésimo sétimo',
						'38'	=>	'Trigésimo oitavo',
						'39'	=>	'Trigésimo nono',
						'40'	=>	'Quadragésimo',
						'41'	=>	'Quadragésimo primeiro',
						'42'	=>	'Quadragésimo segundo',
						'43'	=>	'Quadragésimo terceiro',
						'44'	=>	'Quadragésimo quarto',
						'45'	=>	'Quadragésimo quinto',
						'46'	=>	'Quadragésimo sexto',
						'47'	=>	'Quadragésimo sétimo',
						'48'	=>	'Quadragésimo oitavo',
						'49'	=>	'Quadragésimo nono',
						'50'	=>	'Quinqüagésimo',
/****************************
 END TRANSLATION
****************************/
			);
	}
}
?>